@@ -9,28 +9,19 @@ from neo4j import GraphDatabase
 
 from .... schema import TriplesQueryRequest, TriplesQueryResponse, Error
 from .... schema import Value, Triple
-from .... schema import triples_request_queue
-from .... schema import triples_response_queue
-from .... base import ConsumerProducer
+from .... base import TriplesQueryService
 
-module = "triples-query"
-
-default_input_queue = triples_request_queue
-default_output_queue = triples_response_queue
-default_subscriber = module
+default_ident = "triples-query"
 
 default_graph_host = 'bolt://neo4j:7687'
 default_username = 'neo4j'
 default_password = 'password'
 default_database = 'neo4j'
 
-class Processor(ConsumerProducer):
+class Processor(TriplesQueryService):
 
     def __init__(self, **params):
 
-        input_queue = params.get("input_queue", default_input_queue)
-        output_queue = params.get("output_queue", default_output_queue)
-        subscriber = params.get("subscriber", default_subscriber)
         graph_host = params.get("graph_host", default_graph_host)
         username = params.get("username", default_username)
         password = params.get("password", default_password)
@@ -38,12 +29,9 @@ class Processor(ConsumerProducer):
 
         super(Processor, self).__init__(
             **params | {
-                "input_queue": input_queue,
-                "output_queue": output_queue,
-                "subscriber": subscriber,
-                "input_schema": TriplesQueryRequest,
-                "output_schema": TriplesQueryResponse,
                 "graph_host": graph_host,
+                "username": username,
+                "database": database,
             }
         )
 
@@ -58,44 +46,37 @@ class Processor(ConsumerProducer):
         else:
             return Value(value=ent, is_uri=False)
 
-    async def handle(self, msg):
+    async def query_triples(self, query):
 
         try:
 
-            v = msg.value()
-
-            # Sender-produced ID
-            id = msg.properties()["id"]
-
-            print(f"Handling input {id}...", flush=True)
-
             triples = []
 
-            if v.s is not None:
-                if v.p is not None:
-                    if v.o is not None:
+            if query.s is not None:
+                if query.p is not None:
+                    if query.o is not None:
 
                         # SPO
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel {uri: $rel}]->(dest:Literal {value: $value}) "
                             "RETURN $src as src",
-                            src=v.s.value, rel=v.p.value, value=v.o.value,
+                            src=query.s.value, rel=query.p.value, value=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
-                            triples.append((v.s.value, v.p.value, v.o.value))
+                            triples.append((query.s.value, query.p.value, query.o.value))
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel {uri: $rel}]->(dest:Node {uri: $uri}) "
                             "RETURN $src as src",
-                            src=v.s.value, rel=v.p.value, uri=v.o.value,
+                            src=query.s.value, rel=query.p.value, uri=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
-                            triples.append((v.s.value, v.p.value, v.o.value))
+                            triples.append((query.s.value, query.p.value, query.o.value))
 
                     else:
 
@@ -104,52 +85,52 @@ class Processor(ConsumerProducer):
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel {uri: $rel}]->(dest:Literal) "
                             "RETURN dest.value as dest",
-                            src=v.s.value, rel=v.p.value,
+                            src=query.s.value, rel=query.p.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((v.s.value, v.p.value, data["dest"]))
+                            triples.append((query.s.value, query.p.value, data["dest"]))
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel {uri: $rel}]->(dest:Node) "
                             "RETURN dest.uri as dest",
-                            src=v.s.value, rel=v.p.value,
+                            src=query.s.value, rel=query.p.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((v.s.value, v.p.value, data["dest"]))
+                            triples.append((query.s.value, query.p.value, data["dest"]))
 
                 else:
 
-                    if v.o is not None:
+                    if query.o is not None:
 
                         # SO
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel]->(dest:Literal {value: $value}) "
                             "RETURN rel.uri as rel",
-                            src=v.s.value, value=v.o.value,
+                            src=query.s.value, value=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((v.s.value, data["rel"], v.o.value))
+                            triples.append((query.s.value, data["rel"], query.o.value))
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel]->(dest:Node {uri: $uri}) "
                             "RETURN rel.uri as rel",
-                            src=v.s.value, uri=v.o.value,
+                            src=query.s.value, uri=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((v.s.value, data["rel"], v.o.value))
+                            triples.append((query.s.value, data["rel"], query.o.value))
 
                     else:
 
@@ -158,55 +139,55 @@ class Processor(ConsumerProducer):
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel]->(dest:Literal) "
                             "RETURN rel.uri as rel, dest.value as dest",
-                            src=v.s.value,
+                            src=query.s.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((v.s.value, data["rel"], data["dest"]))
+                            triples.append((query.s.value, data["rel"], data["dest"]))
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node {uri: $src})-[rel:Rel]->(dest:Node) "
                             "RETURN rel.uri as rel, dest.uri as dest",
-                            src=v.s.value,
+                            src=query.s.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((v.s.value, data["rel"], data["dest"]))
+                            triples.append((query.s.value, data["rel"], data["dest"]))
 
 
             else:
 
-                if v.p is not None:
+                if query.p is not None:
 
-                    if v.o is not None:
+                    if query.o is not None:
 
                         # PO
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node)-[rel:Rel {uri: $uri}]->(dest:Literal {value: $value}) "
                             "RETURN src.uri as src",
-                            uri=v.p.value, value=v.o.value,
+                            uri=query.p.value, value=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((data["src"], v.p.value, v.o.value))
+                            triples.append((data["src"], query.p.value, query.o.value))
 
                         records, summary, keys = self.io.execute_query(
-                            "MATCH (src:Node)-[rel:Rel {uri: $uri}]->(dest:Node {uri: $uri}) "
+                            "MATCH (src:Node)-[rel:Rel {uri: $uri}]->(dest:Node {uri: $dest}) "
                             "RETURN src.uri as src",
-                            uri=v.p.value, dest=v.o.value,
+                            uri=query.p.value, dest=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((data["src"], v.p.value, v.o.value))
+                            triples.append((data["src"], query.p.value, query.o.value))
 
                     else:
 
@@ -215,52 +196,52 @@ class Processor(ConsumerProducer):
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node)-[rel:Rel {uri: $uri}]->(dest:Literal) "
                             "RETURN src.uri as src, dest.value as dest",
-                            uri=v.p.value,
+                            uri=query.p.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((data["src"], v.p.value, data["dest"]))
+                            triples.append((data["src"], query.p.value, data["dest"]))
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node)-[rel:Rel {uri: $uri}]->(dest:Node) "
                             "RETURN src.uri as src, dest.uri as dest",
-                            uri=v.p.value,
+                            uri=query.p.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((data["src"], v.p.value, data["dest"]))
+                            triples.append((data["src"], query.p.value, data["dest"]))
 
                 else:
 
-                    if v.o is not None:
+                    if query.o is not None:
 
                         # O
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node)-[rel:Rel]->(dest:Literal {value: $value}) "
                             "RETURN src.uri as src, rel.uri as rel",
-                            value=v.o.value,
+                            value=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((data["src"], data["rel"], v.o.value))
+                            triples.append((data["src"], data["rel"], query.o.value))
 
                         records, summary, keys = self.io.execute_query(
                             "MATCH (src:Node)-[rel:Rel]->(dest:Node {uri: $uri}) "
                             "RETURN src.uri as src, rel.uri as rel",
-                            uri=v.o.value,
+                            uri=query.o.value,
                             database_=self.db,
                         )
 
                         for rec in records:
                             data = rec.data()
-                            triples.append((data["src"], data["rel"], v.o.value))
+                            triples.append((data["src"], data["rel"], query.o.value))
 
                     else:
 
@@ -295,37 +276,17 @@ class Processor(ConsumerProducer):
                 for t in triples
             ]
 
-            print("Send response...", flush=True)
-            r = TriplesQueryResponse(triples=triples, error=None)
-            await self.send(r, properties={"id": id})
-
-            print("Done.", flush=True)
+            return triples
 
         except Exception as e:
 
             print(f"Exception: {e}")
-
-            print("Send error response...", flush=True)
-
-            r = TriplesQueryResponse(
-                error=Error(
-                    type = "llm-error",
-                    message = str(e),
-                ),
-                response=None,
-            )
-
-            await self.send(r, properties={"id": id})
-
-            self.consumer.acknowledge(msg)
+            raise e
             
     @staticmethod
     def add_args(parser):
 
-        ConsumerProducer.add_args(
-            parser, default_input_queue, default_subscriber,
-            default_output_queue,
-        )
+        TriplesQueryService.add_args(parser)
 
         parser.add_argument(
             '-g', '--graph-host',
@@ -353,5 +314,5 @@ class Processor(ConsumerProducer):
 
 def run():
 
-    Processor.launch(module, __doc__)
+    Processor.launch(default_ident, __doc__)
 
